// Copyright 2003 Adam Megacz, see the COPYING file for licensing [GPL] 

package org.xwt.js; 
import org.xwt.util.*; 
import java.io.*;
import java.util.*;

/** Implementation of a JavaScript Scope */
class ScopeImpl extends JS.Obj { 
    private JS.Scope parentScope;
    private static final Object NULL_PLACEHOLDER = new Object();
    public ScopeImpl(JS.Scope parentScope, boolean sealed) {
        super(sealed);
        if (parentScope == this) throw new Error("can't make a scope its own parent!");
        this.parentScope = parentScope;
    }
    public Object[] keys() { throw new Error("you can't enumerate the properties of a Scope"); }
    public boolean has(Object key) { return super.get(key) != null; }
    // we use _get instead of get solely to work around a GCJ bug
    public Object _get(Object key) {
        Object o = super.get(key);
        if (o != null) return o == NULL_PLACEHOLDER ? null : o;
        else return parentScope == null ? null : parentScope.get(key);
    }
    // we use _put instead of put solely to work around a GCJ bug
    public void _put(Object key, Object val) {
        if (parentScope != null && !has(key)) parentScope.put(key, val);
        else super.put(key, val == null ? NULL_PLACEHOLDER : val);
    }
    public boolean isTransparent() { return false; }
    public void declare(String s) {
        if (isTransparent()) parentScope.declare(s);
        else super.put(s, NULL_PLACEHOLDER);
    }
    public Scope getParentScope() { return parentScope; }
} 
